#========================================================================================
# package      DE_EPAGES::Content::Hooks::Servlet
# state        public
# description  Additional hook function for OnIsEp6HandledUnityRequest hook.
#========================================================================================
package DE_EPAGES::Content::Hooks::Servlet;

use strict;

#========================================================================================
# function     OnIsEp6HandledUnityRequest
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnIsEp6HandledUnityRequest($hParams);
#----------------------------------------------------------------------------------------
# description  Triggered by hook IsEp6HandledUnityRequest.
#               Legacy category links in a unity shop should be rendered as a deep redirect
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Object - servlet object</li>
#                   <li>Values - attributes</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnIsEp6HandledUnityRequest {
    my ($hParams) = @_;
    my $Servlet = $hParams->{'Object'};

    # skip if another hook function already acknowledges the request as a unity request handled by epages6
    return if $hParams->{'Values'}->{'IsEp6HandledUnityRequest'};

    my $ViewAction = $Servlet->params('ViewAction');
    if ($ViewAction eq 'View' || $ViewAction eq 'MobileSF-View' || $ViewAction eq 'ViewPopup') {
        my $ViewObject = $Servlet->object();
        $hParams->{'Values'}->{'IsEp6HandledUnityRequest'} = 1 if $ViewObject->instanceOf('ContentObject');
    }

    return;
}

1;
