#==============================================================================
# package      DE_EPAGES::Content::UI::Category
# state        private
#==============================================================================
package DE_EPAGES::Content::UI::Category;
use base DE_EPAGES::Content::UI::ContentObject;
use strict;

use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );

sub Save {
    my $self = shift;
    my $Servlet = shift;

    # save URL image
    my $Category = $Servlet->object;
    my $Image = $Servlet->params->{'Image'};
    if (defined $Image && $Image =~ /\/\//) {
        $Category->set({'Image' => $Image});
    }

    # save the main content of the category
    $self->SUPER::Save($Servlet);

    return;
}

sub SaveUnity {
    my $self = shift;
    my ($Servlet) = @_;

    # check form values
    my $Category = $Servlet->object;
    $Servlet->form->form($Category, 'UnityCategory');

    $self->Save($Servlet);

    return;
}

sub ViewCached {
    my $self = shift;
    my ($Servlet) = @_;

    if ($Servlet->site->class->alias eq 'Shop'
        and $Servlet->site->get('IsUnityRegistered')) {
        #unity view -> redirect to epages-ui but modify target url to include language
        my $LocaleID = $Servlet->locale();
        my ($LanguageCode2) = ($LocaleID =~ /^(\w{2})/);
        my $TargetUrl = $Servlet->object->get('WebUrlUnitySSL');
        $TargetUrl =~ s/^(.*)([\/]c[\/].*)$/$1\/$LanguageCode2$2/;
        return $Servlet->redirect($TargetUrl)
    } else {

        my $Page = $Servlet->params('Page');
        my $NotFirstPage = (defined $Page and $Page != 1);

        $Servlet->vars('NotFirstPage', $NotFirstPage);

        return $self->SUPER::ViewCached($Servlet);
    }
}

#========================================================================================
# function     SaveGUIDList
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveGUIDList( $Servlet );
#----------------------------------------------------------------------------------------
# description  The ChangeAction SaveGUIDList is wrapped in order to make the fact
#               more transparent that Catrgory objectswill use other implementations of
#               - deleteObjectListWrapper
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SaveGUIDList {
    my $self = shift;
    my ($Servlet) = @_;
    return $self->SUPER::SaveGUIDList($Servlet);
}

#========================================================================================
# function     deleteObjectListWrapper
# state        protected
#----------------------------------------------------------------------------------------
# syntax       $self->deleteObjectListWrapper( $aListObjects );
#----------------------------------------------------------------------------------------
# description  Delete the Objects from the list
#               run the BatchProcess on the right context
#----------------------------------------------------------------------------------------
# input        $aListObjects | object list | object
#========================================================================================
sub deleteObjectListWrapper {
    my $self = shift;
    my ( $aListObjects ) = @_;

    RunBatchProcess(['DeleteCategories'], sub {
        return $self->SUPER::deleteObjectListWrapper($aListObjects);
    });
}

1;
