#==============================================================================
# package      DE_EPAGES::PayPalExpress::UI::BasketOrder
# state        private
#==============================================================================
package DE_EPAGES::PayPalExpress::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;
use DE_EPAGES::ExternalPayment::API::Log qw ( LogPayment );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;

use strict;

#==============================================================================
sub ViewMultiPaymentPayPalExpress {
    my $self = shift;
    my ($Servlet) = @_;
    my $BasketOrder = $Servlet->object;

    my $pli = $BasketOrder->container->get('Payment');
    return unless (defined($pli) && $pli->instanceOf('LineItemPaymentPayPalExpress'));

    # start standard paypal payment
    my $redirectUrl = $pli->checkoutRequest($Servlet, 0);
    $Servlet->vars('PayPalExpressRedirectUrl', $redirectUrl);

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess', $BasketOrderProcess->tleHash);

    return;
}

sub ViewPayPalExpressFailed {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrder = $Servlet->object();

    my $PayPalErrorCode = $Servlet->vars('PPEErrorCode');

    # On error 10486 redirect back to PayPal
    # https://epages.atlassian.net/browse/EPG-33597
    if ($PayPalErrorCode && $PayPalErrorCode eq '10486') {
        $self->_redirectToPayPalExpress($Servlet, $BasketOrder);
        return;
    }

    my $Shop = $BasketOrder->get('Shop');

    if ($Shop->isUnity()) {
        my $LineItemPayment = $BasketOrder->container()->get('Payment');
        $LineItemPayment->set({'PaymentErrorCode' => 'Failed'});
        $Servlet->vars('ViewObjectID', $BasketOrder->id());
        $Servlet->vars('ViewAction', 'ViewMultiCheckoutCustomerOrder');
    }
}

sub _redirectToPayPalExpress {
    my $self = shift;
    my ($Servlet, $BasketOrder) = @_;

    my $LineItemPayment = $BasketOrder->container()->get('Payment');

    my $PayPalUrl = $LineItemPayment->get('IsTestTransaction') ? 
        'PayPalExpress_SandboxURL_Redirect' : 'PayPalExpress_ProductionURL_Redirect';

    my $PayPalRedirectUrl = LoadRootObject()->get($PayPalUrl);
    $PayPalRedirectUrl .= $LineItemPayment->get('Token');

    LogPayment(
        'PayPalExpress',
        'ViewPayPalExpressFailed Error code 10486 redirecting customer to PayPal',
        {'OrderID' => $BasketOrder->id}
    );

    $Servlet->vars('ExitAfterEvent', 1);
    $Servlet->vars('ResponseContentProcessed', 1);

    my $Response = $Servlet->response();
    $Response->code(302);
    $Response->header('Location', $PayPalRedirectUrl);

    return;
}

1;

