#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::Request_URLBuilder
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for URLBuilder.pm
#========================================================================================
package DE_EPAGES::PayPalExpress::t::unit::Request_URLBuilder;
use base DE_EPAGES::Core::API::Test;
use DE_EPAGES::PayPalExpress::API::Request::URLBuilder;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use strict;

sub test {
    my $self = shift;

    $self->test_getReturnURL_Basket();
    $self->test_getReturnURL_CustomerOrder();
    $self->test_getCancelURLCustomerOrder();
    $self->test_getCancelURLBasket();
    $self->test_getGiroPaySuccessURLCustomerOrder();
    $self->test_getGiroPaySuccessURLBasket();
    $self->test_getGiroPayCancelURL();
    $self->test_getCallbackURL_CustomerOrder();
    $self->test_getCallbackURL_Basket();

    done_testing();
}

#----------------------------------------------------------------------------------------
sub test_getReturnURL_Basket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $aExpectedParameters = [
            { 'object' => 'DE_EPAGES::Order::API::Object::Basket' },
            { 'hashref' => {'ChangeAction' => ['BlockMonitor', 'PayPalExpressReturn'] } },
            { 'hashref' => {'UseSSL' => 1, 'AddToken' => 1, 'Type' => 'sf/mobile'} },
            { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' },
    ];
    my $URLBuilder = $self->_buildURLBuilder( $aExpectedParameters );
    my $Basket = $self->_createBasket();

    my $ReturnURL = $URLBuilder->getReturnURL( $Basket );

    is( $ReturnURL, 'dummyURL', "$SubTestName - test if the correct URL was returned" );

    return;
}

#----------------------------------------------------------------------------------------
sub test_getReturnURL_CustomerOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $aExpectedParameters = [
            { 'object' => 'DE_EPAGES::Order::API::Object::CustomerOrder' },
            { 'hashref' => {'ChangeAction' => ['BlockMonitor', 'PayPalExpressReturn'] } },
            { 'hashref' => {'UseSSL' => 1, 'AddToken' => 1, 'Type' => 'sf/mobile'} },
            { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' },
    ];
    my $URLBuilder = $self->_buildURLBuilder( $aExpectedParameters );
    my $CustomerOrder = $self->_createCustomerOrder();

    my $ReturnURL = $URLBuilder->getReturnURL( $CustomerOrder );

    is( $ReturnURL, 'dummyURL', "$SubTestName - test if the correct URL was returned" );

    return;
}

#----------------------------------------------------------------------------------------
sub test_getCancelURLCustomerOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $aExpectedParameters = [
            { 'object' => 'DE_EPAGES::Order::API::Object::CustomerOrder' },
            { 'hashref' => {'ChangeAction' => ['BlockMonitor', 'PayPalExpressCancelOnOrder'] } },
            { 'hashref' => {'UseSSL' => 1, 'AddToken' => 1, 'Type' => 'sf/mobile'} },
            { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' },
    ];
    my $URLBuilder = $self->_buildURLBuilder( $aExpectedParameters );
    my $CustomerOrder = $self->_createCustomerOrder();

    my $ReturnURL = $URLBuilder->getCancelURLCustomerOrder( $CustomerOrder );

    is( $ReturnURL, 'dummyURL', "$SubTestName - test if the correct URL was returned" );

    return;
}

#----------------------------------------------------------------------------------------
sub test_getCancelURLBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $aExpectedParameters = [
            { 'object' => 'DE_EPAGES::Order::API::Object::Basket' },
            { 'hashref' => {'ChangeAction' => ['BlockMonitor', 'PayPalExpressCancel'] } },
            { 'hashref' => {'UseSSL' => 1, 'AddToken' => 1, 'Type' => 'sf/mobile'} },
            { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' },
    ];
    my $URLBuilder = $self->_buildURLBuilder( $aExpectedParameters );
    my $Basket = $self->_createBasket();

    my $ReturnURL = $URLBuilder->getCancelURLBasket( $Basket );

    is( $ReturnURL, 'dummyURL', "$SubTestName - test if the correct URL was returned" );

    return;
}

#----------------------------------------------------------------------------------------
sub test_getGiroPaySuccessURLCustomerOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $aExpectedParameters = [
            { 'object' => 'DE_EPAGES::Order::API::Object::CustomerOrder' },
            { 'hashref' => {'ChangeAction' => 'BlockMonitor'} },
            { 'hashref' => {'UseSSL' => 1, 'AddToken' => 1, 'Type' => 'sf/mobile'} },
            { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' },
    ];
    my $URLBuilder = $self->_buildURLBuilder( $aExpectedParameters );
    my $CustomerOrder = $self->_createCustomerOrder();

    my $ReturnURL = $URLBuilder->getGiroPaySuccessURLCustomerOrder( $CustomerOrder );

    is( $ReturnURL, 'dummyURL', "$SubTestName - test if the correct URL was returned" );

    return;
}

#----------------------------------------------------------------------------------------
sub test_getGiroPaySuccessURLBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GUID = 'GUID_1a2b3d4e';
    my $aExpectedParameters = [
            { 'object' => 'DE_EPAGES::Order::API::Object::Basket' },
            { 'hashref' => {'ChangeAction' => ['BlockMonitor', 'PayPalExpressGiroPaySuccess'], 'ppeguid' => $GUID} },
            { 'hashref' => {'UseSSL' => 1, 'AddToken' => 1, 'Type' => 'sf/mobile'} },
            { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' },
    ];
    my $URLBuilder = $self->_buildURLBuilder( $aExpectedParameters );
    my $Basket = $self->_createBasket();

    my $ReturnURL = $URLBuilder->getGiroPaySuccessURLBasket( $Basket, $GUID );

    is( $ReturnURL, 'dummyURL', "$SubTestName - test if the correct URL was returned" );

    return;
}

#----------------------------------------------------------------------------------------
sub test_getGiroPayCancelURL {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GUID = 'GUID_1a2b3d4e';
    my $aExpectedParameters = [
            { 'object' => 'DE_EPAGES::Shop::API::Object::Shop' },
            { 'hashref' => {'ChangeAction' => ['BlockMonitor', 'PayPalExpressGiroPayCancel'], 'ppeguid' => $GUID} },
            { 'hashref' => {'UseSSL' => 1, 'AddToken' => 1, 'Type' => 'sf/mobile'} },
            { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' },
    ];
    my $URLBuilder = $self->_buildURLBuilder( $aExpectedParameters );
    my $Shop = $self->_createShop();

    my $ReturnURL = $URLBuilder->getGiroPayCancelURL( $Shop, $GUID );

    is( $ReturnURL, 'dummyURL', "$SubTestName - test if the correct URL was returned" );

    return;
}

#----------------------------------------------------------------------------------------
sub test_getCallbackURL_CustomerOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GUID = 'GUID_1a2b3d4e';
    my $aExpectedParameters = [
            { 'object' => 'DE_EPAGES::Order::API::Object::CustomerOrder' },
            { 'hashref' => { 'ChangeAction' => ['BlockMonitor', 'PayPalExpressCallback'], 'BasketGUID' => $GUID } },
            { 'hashref' => {'UseSSL' => 1, 'AddToken' => 1, 'Type' => 'sf/mobile'} },
            { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' },
    ];
    my $URLBuilder = $self->_buildURLBuilder( $aExpectedParameters );
    my $CustomerOrder = $self->_createCustomerOrder();

    my $ReturnURL = $URLBuilder->getCallbackURL( $CustomerOrder, $GUID );

    is( $ReturnURL, 'dummyURL', "$SubTestName - test if the correct URL was returned" );

    return;
}

#----------------------------------------------------------------------------------------
sub test_getCallbackURL_Basket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $GUID = 'GUID_1a2b3d4e';
    my $aExpectedParameters = [
            { 'object' => 'DE_EPAGES::Order::API::Object::Basket' },
            { 'hashref' => { 'ChangeAction' => ['BlockMonitor', 'PayPalExpressCallback'], 'BasketGUID' => $GUID } },
            { 'hashref' => {'UseSSL' => 1, 'AddToken' => 1, 'Type' => 'sf/mobile'} },
            { 'object' => 'DE_EPAGES::Presentation::Servlet::Permission' },
    ];
    my $URLBuilder = $self->_buildURLBuilder( $aExpectedParameters );
    my $Basket = $self->_createBasket();

    my $ReturnURL = $URLBuilder->getCallbackURL( $Basket, $GUID );

    is( $ReturnURL, 'dummyURL', "$SubTestName - test if the correct URL was returned" );

    return;
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildURLBuilder {
    my $self = shift;
    my ( $aExpectedParameters ) = @_;

    my $Servlet = $self->_createServlet();
    my $aParameterList = [$Servlet];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
                   'DE_EPAGES::PayPalExpress::API::Request::URLBuilder',
                   $aParameterList
              );
    $Mockify->addMockWithReturnValueAndParameterCheck( '_BuildShopUrl', 'dummyURL', $aExpectedParameters );

    return $Mockify->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createServlet {
    my $self = shift;

    my $hAtributes = {};
    my $Servlet = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Presentation::Servlet::Permission', $hAtributes );

    return $Servlet;
}

#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;

    my $hAtributes = {};
    my $name = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', $hAtributes );

    return $name;
}

#----------------------------------------------------------------------------------------
sub _createCustomerOrder {
    my $self = shift;

    my $hAtributes = {};
    my $name = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::CustomerOrder', $hAtributes );

    return $name;
}

#----------------------------------------------------------------------------------------
sub _createShop {
    my $self = shift;

    my $hAtributes = {};
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hAtributes );

    return $Shop;
}
__PACKAGE__->runScript();