#========================================================================================
# package      DE_EPAGES::PayPalExpress::t::unit::UI_Order
# base          DE_EPAGES::Test::API::UnitTest::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Order
#========================================================================================
package main;
use base DE_EPAGES::Test::API::UnitTest::Test;
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::PayPalExpress::UI::Order;
use Test::Mockify::Matcher qw ( String HashRef );
use Test::More;
use Test::Mockify::Sut;

use strict;

sub test {
    my $self = shift;

    $self->_buildMocks();

    $self->transactionShouldBeSetToCancel_PayPalExpressCancelOnOrder();

    done_testing();
}
#----------------------------------------------------------------------------------------
sub transactionShouldBeSetToCancel_PayPalExpressCancelOnOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Servlet = $self->memory()->get('Servlet');
    my $SUT = $self->memory()->get('SUT');

    $SUT->PayPalExpressCancelOnOrder($Servlet);

    my $LineItemContainer = $self->memory()->get('LineItemContainer');
    my $LineItemPayment = $LineItemContainer->get('Payment');

    is($LineItemPayment->get('TransStatus'), 'cancelled',"$SubTestName - Should transaction status set to cancelled");
    is($LineItemPayment->get('TransTime'), 'NewDateTime',"$SubTestName - Should transaction time set to NewDateTime");
}
#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------
sub _buildMocks {
    my $self = shift;

    my $SUT = $self->_createSUT();

    my $Shop = $self->mockFactory()->buildShop({
            'IsUnityRegistered' => 0
        })->getMockObject();

    my $LineItemPaymentPayPalExpress = $self->mockFactory->build('DE_EPAGES::PayPalExpress::API::Object::LineItemPaymentPayPalExpress',
                         {'TransID' => 0, 'TransStatus' => undef, 'TransTime' => 'OldDateTime'});
    $LineItemPaymentPayPalExpress->addMockWithReturnValue('_GetCurrentDBHandle', $self->_mockCurrentDBHandle());

    my $LineItemContainer = $self->mockFactory->buildLineItemContainer({
            'Payment' => $LineItemPaymentPayPalExpress->getMockObject()
    })->getMockObject();

    my $OrderBuild = $self->mockFactory->buildCustomerOrder({
        'Shop'              => $Shop,
        'LineItemContainer' => $LineItemContainer,
        'ID'                => '12345'
    });
    $OrderBuild->addMockWithReturnValue('pathString', '/path/to/order');
    my $Order = $OrderBuild->getMockObject();

    my $Servlet = $self->mockFactory()->buildServlet($Order)->getMockObject();

    $self->memory()->add('LineItemContainer', $LineItemContainer);
    $self->memory()->add('Order', $Order);
    $self->memory()->add('Servlet', $Servlet);
    $self->memory()->add('Shop', $Shop);
    $self->memory()->add('SUT', $SUT);
}
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $aParameterList = [];
    my $MockifySut = Test::Mockify::Sut->new(
        'DE_EPAGES::PayPalExpress::UI::Order',
        $aParameterList
    );
    $MockifySut->mockImported('DE_EPAGES::ExternalPayment::API::Log', 'LogPayment')
        ->when(String(), String(), HashRef())->thenReturnUndef();

    return $MockifySut->getMockObject();
}

sub _mockCurrentDBHandle {
    my $self = shift;

    my $MockedDBI = Test::MockObject->new();
    $MockedDBI->mock('currentDateTime', sub {
        return 'NewDateTime';
    });

    return $MockedDBI;
}

__PACKAGE__->runScript();