#==============================================================================
# package      DE_EPAGES::PriceList::UI::PriceList
# state        private
#==============================================================================
package DE_EPAGES::PriceList::UI::PriceList;
use base qw( DE_EPAGES::Presentation::UI::Object );

use strict;
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError Error);
use DE_EPAGES::Core::API::PerlTools qw (ElementOf );
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias Transaction);

#==============================================================================
# Event handlers
#==============================================================================
sub attributeValues {
    my $self = shift;
    my ( $Servlet, $Object, $Class ) = @_;

    my $hhValues = $self->SUPER::attributeValues( $Servlet, $Object, $Class );

    return $hhValues if grep { $_->{'Reason'} =~ /DATETIME/ }  @{$Servlet->form->formErrors};

    my $FormValues = $hhValues->{'Values'};
    if( defined $FormValues->{'ValidTo'} &&
        defined $FormValues->{'ValidFrom'} &&
        $FormValues->{'ValidTo'} <= $FormValues->{'ValidFrom'})
    {
        $Servlet->form->addFormError({
            'Reason'=> 'InvalidTimeFrame',
            'Name'  => 'ValidFrom',
        }) ;
        $Servlet->form->addFormError({
            'Reason'=> 'InvalidTimeFrame',
            'Name'  => 'ValidTo',
        }) ;
    }
    $Servlet->form->executeFormError;

    # don't test if a new price list should created
    return $hhValues unless $Object->instanceOf('PriceList');

    my $PriceList = $Object;
    my $OldFrom = $PriceList->get('ValidFrom');
    my $OldTo   = $PriceList->get('ValidTo');
    unless ($FormValues->{'ValidFrom'} == $OldFrom and $FormValues->{'ValidTo'} == $OldTo) {
        eval {
            $PriceList->checkCanSetValidDates($FormValues);
        };
        if (ExistsError()) {
            my $Error = GetError();
            my $hErrorVars;
            if ($Error->code eq 'CustomerGroupAssignedToAnotherPriceListAtThisTimeFrame') {
                $hErrorVars = {
                    'Reason'=> 'NotForAllCustomerGroupsIsPossibleToChangeToThisTimeFrame',
                    'CustomerGroup' => $Error->vars->{'CustomerGroup'}->tleHash,
                    'PriceList' => $Error->vars->{'AssignedPriceList'}->tleHash,
                };
            } elsif ($Error->code eq 'CustomerAssignedToAnotherPriceListAtThisTimeFrame') {
                $hErrorVars = {
                    'Reason'=> 'NotForAllCustomersIsPossibleToChangeToThisTimeFrame',
                    'Customer' => $Error->vars->{'Customer'}->tleHash,
                    'PriceList' => $Error->vars->{'AssignedPriceList'}->tleHash,
                };
            }
            $Servlet->form->addFormError({
                'Name'  => 'ValidFrom',
                %$hErrorVars,
            }) ;
            # add a second error with ValidTo (to mark the second input field)
            # use no reason, to prevent from displaying error reasons twice
            $Servlet->form->addFormError({
                'Name'  => 'ValidTo',
                'Reason' => 'dummy',
            }) ;
            $Servlet->form->executeFormError();
        }
    } # else {everything like before}

    return $hhValues;
}

sub NewPriceList {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Shop    = $Servlet->object;

    my $Folder  = $Shop->folder('PriceLists', { 'Position'=> 30 });
    my $Class   = LoadClassByAlias ('PriceList');
    return $self->SUPER::NewObject($Servlet, $Folder, $Class);
}

sub AddCustomerGroup {
    my $self = shift;
    my ($Servlet) = @_;

    my $PriceList = $Servlet->object;
    return unless $Servlet->params('AddCustomerGroupID');

    my $Shop = $PriceList->getSite();
    my $CustomerGroup = $Shop->loadObject( $Servlet->params('AddCustomerGroupID'), 'CustomerGroup' );
    my $Form = $Servlet->form;

    eval {
        $PriceList->addCustomerGroup($CustomerGroup);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'CustomerGroupAssignedToAnotherPriceListAtThisTimeFrame';
        $Form->mapFormError({'Name' => 'AddCustomerGroupID'}, $Error);
    }

    $Form->executeFormError;
    $Shop->invalidatePageCache;

    return;
}

sub RemoveCustomerGroups {
    my $self = shift;
    my ($Servlet) = @_;

    my $PriceList = $Servlet->object;
    my $Shop = $PriceList->getSite;
    my @CustomerGroupIDs = $Servlet->form->value('ListObjectID');

    $PriceList->removeCustomerGroup($_) foreach @{ $Shop->loadObjects(\@CustomerGroupIDs, 'CustomerGroup') };
    $Shop->invalidatePageCache;

    return;
}


sub AddCustomerListFromTray {
    my $self = shift;
    my ($Servlet) = @_;

    my $PriceList = $Servlet->object;

    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    my $Objects = $Tray->objectsByClass(LoadClassByAlias('Customer'));
    my $Form = $Servlet->form;

    foreach my $Customer (@$Objects) {
        eval {
            $PriceList->addCustomer($Customer);
        };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) unless $Error->code eq 'CustomerAssignedToAnotherPriceListAtThisTimeFrame';
            $Form->mapFormError({'Name' => 'AddCustomerListFromTray'}, $Error);
        }
    }
    $Form->executeFormError;
    return;
}

sub RemoveCustomers {
    my $self = shift;
    my ($Servlet) = @_;

    my $PriceList = $Servlet->object;
    my $Shop = $PriceList->getSite;
    my @CustomerIDs = $Servlet->form->value('ListObjectID');

    $PriceList->removeCustomer($_) foreach @{ $Shop->loadObjects(\@CustomerIDs, 'Customer') };
    return;
}

sub SaveGlobalValueDiscounts {
    my $self = shift;
    my ($Servlet) = @_;

    # new value discounts
    if ( $Servlet->params('NewDiscount') !~ /^\s*$/ or $Servlet->params('NewDiscountAbsolute') !~ /^\s*$/ or $Servlet->params('NewValue') !~ /^\s*$/){
        NewGlobalValueDiscounts($self, $Servlet);
    }
    my $PriceList = $Servlet->object;
    my $Class = LoadClassByAlias('GlobalValueDiscount');
    # save value discounts
    my $FormValues = $Servlet->form->form($PriceList, 'SaveGlobalValueDiscounts');
    my $Index = 0;
    foreach my $Entry (@{$FormValues->{'GlobalValueDiscounts'}}) {
        my $GlobalValueDiscount = $Class->loadObject($Entry->{'GlobalValueDiscountID'});

        # security check
        die 'GlobalValueDiscount from different PriceList' if $PriceList->id != $GlobalValueDiscount->get('PriceList')->id;

        delete $Entry->{'Discount'} if defined $Entry->{'Discount'} and $Entry->{'Discount'} !~ /\S/;
        delete $Entry->{'DiscountAbsolute'} if defined $Entry->{'DiscountAbsolute'} and $Entry->{'DiscountAbsolute'} !~ /\S/;
        if (defined  $GlobalValueDiscount) {
            if (defined $Entry->{'Discount'} || defined $Entry->{'DiscountAbsolute'}) {
                # test absolute discount > interval
                if (defined $Entry->{'DiscountAbsolute'}) {
                    my $DiscountAbsolute = $Entry->{'DiscountAbsolute'};
                    $Servlet->form->executeFormError({
                        'Reason'=> 'WRONG_DISCOUNT',
                        'Name'  => 'DiscountAbsolute',
                        'Form'  => 'SaveGlobalValueDiscounts',
                        'LoopName' => 'GlobalValueDiscounts',
                        'Index' => $Index,
                    }) if $DiscountAbsolute > $GlobalValueDiscount->get('Value');
                }
                $GlobalValueDiscount->set({'Discount'=>$Entry->{'Discount'}, 'DiscountAbsolute' => $Entry->{'DiscountAbsolute'} });
            } else {
                $GlobalValueDiscount->delete;
            }
        }
        $Index++;
    }
    return;

}

sub NewGlobalValueDiscounts {
    my $self = shift;
    my ($Servlet) = @_;

    my $PriceList   = $Servlet->object;

    my $Shop        = $PriceList->getSite();
    my $FormValues  = $Servlet->form->form($PriceList, 'NewGlobalValueDiscount');
    my $Value       = $Shop->formatter($PriceList->get('CurrencyID'))->round($FormValues->{'NewValue'});

    #test for discount or absolute discount
    unless (defined($FormValues->{'NewDiscountAbsolute'}) || defined($FormValues->{'NewDiscount'})) {
        $Servlet->form->addFormError({
            'Reason'=> 'MISSING_DISCOUNT',
            'Name'  => 'NewDiscountAbsolute',
            'Form'  => 'NewGlobalValueDiscount',
        });
        $Servlet->form->addFormError({
            'Reason'=> 'MISSING_DISCOUNT',
            'Name'  => 'NewDiscount',
            'Form'  => 'NewGlobalValueDiscount',
        });
        $Servlet->form->executeFormError();
    }

    #test for existing interval
    my @ExistingGlobalValueDiscounts = grep { $_->get('Value')* 1 == $Value } @{ $PriceList->get('GlobalValueDiscounts') };
    if (scalar @ExistingGlobalValueDiscounts) {
        $Servlet->form->executeFormError({
            'Reason'=> 'INTERVAL_EXISTS',
            'Name'  => 'NewValue',
            'Form'  => 'NewGlobalValueDiscount',
        });
    }

    # test absolute discount > interval
    if (defined $FormValues->{'NewDiscountAbsolute'}) {
        my $NewDiscountAbsolute = $FormValues->{'NewDiscountAbsolute'};
        $Servlet->form->executeFormError({
            'Reason'=> 'WRONG_DISCOUNT',
            'Name'  => 'NewDiscountAbsolute',
            'Form'  => 'NewGlobalValueDiscount',
        }) if $NewDiscountAbsolute > $Value;
    }

    my $Class=LoadClassByAlias('GlobalValueDiscount');

    $Class->insertObject ({
        'PriceList'   => $PriceList,
        'Value'       => $Value,
        'Discount'    => $FormValues->{'NewDiscount'},
        'DiscountAbsolute' => $FormValues->{'NewDiscountAbsolute'},
    });

    return;
}

sub DeleteGlobalValueDiscounts {
    my $self = shift;
    my ($Servlet) = @_;

    # nothing to do
    return if ! $Servlet->params('ListObjectID');

    my $PriceList = $Servlet->object;
    my $Class=LoadClassByAlias('GlobalValueDiscount');
    my $FormValues  = $Servlet->form->form($PriceList, 'DeleteGlobalValueDiscounts');
    my @ListObjects =  map {$Class->loadObject($_->{'ListObjectID'})} @{$FormValues->{'GlobalValueDiscounts'}};

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        $Servlet->vars('DeleteObjects', [ map { $_->tleHash } @ListObjects]);
        $Servlet->vars('DELETE',1);
        return;
    } else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }
    # now delete all objects
    Transaction( sub {
        $_->delete foreach @ListObjects;
    } );
    return;
}

sub AddCustomer {
    my $self = shift;
    my ($Servlet) = @_;

    my $PriceList = $Servlet->object;
    my $Form = $Servlet->form;
    my $Alias  = $Form->value('NewCustomer');
    return unless defined $Alias and $Alias =~ /\S/;

    my $folder = $PriceList->getSite()->folder('Customers');
    my $Customer = $folder->existsChild($Alias) ? $folder->child($Alias) : undef;

    if (not defined $Customer) {
        $Servlet->form->executeFormError({
            'Reason'=> 'ERROR_INVALIDCUSTOMER',
            'Form'  => 'NoForm',
            'Name'  => 'NewCustomer',
        });
    }
    eval {
        $PriceList->addCustomer($Customer);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless ElementOf($Error->code, qw (
            CustomerAssignedToAnotherPriceListAtThisTimeFrame
        ));
        $Form->mapFormError({'Name' => 'AddCustomer'}, $Error);
    }
    $Form->executeFormError;

    return;
}

1;
